@@ -524,6 +524,62 @@ sub is_complete {
     return $self->is_pass || $self->is_fail;
 }
 
+=head2 verify_completion_state
+
+If the user wishes to verify whether a stage completed successfully,
+failed, or should be retried, this attribute can be given a coderef
+to code that checks the result.
+
+The code will be called with the arguments:
+
+    $coderef->( $stats, $stdout, $stderr )
+	    $stats is a hashref containing the accounting info and status of the job
+		$stdout is the pathname of the stdout file
+		$stderr is the pathname of the srderr file
+
+The code can use these (and knowledge provided by the user about the stage
+operation) to test whether the stage succeeded.
+
+The return value from the code should be a string, one of:
+
+    pass
+	fail
+	retry
+
+to classify the result.  If the value is set to 'fail', other
+mechanisms may still apply to change that to 'retry'.
+
+This code attribute could be used for:
+
+- running a program that always gives an error status, but you might wish to treat it as successful
+
+- running a program that can give a zero status but not actually succeed (but perhaps a retry will succeed)
+
+=cut
+
+has 'verify_completion_state' => (
+    is        => 'ro',
+	isa       => 'CodeRef',
+    predicate => '_has_verify_completion_state',
+);
+
+after '_analyse_completion_state' => sub {
+	my $self  = shift;
+	my $run   = shift;
+
+	if ($self->_has_verify_completion_state) {
+		my $stats  = $run->stats;
+		my $stdout = $run->_stdout;
+		my $stderr = $run->_stderr;
+		my $code   = $self->verify_completion_state->( $stats, $stdout, $stderr );
+		if ($code eq 'retry') {
+			$code = 'fail';
+			$run->_analysis_chose_retry(1);
+		}
+		$self->_set_state( $code ) if $code eq 'pass' || $code eq 'fail';
+	}
+};
+
 =head2 should_choose_retry, choose_retries
 
 If a cluster has a tendency to fail randomly, it may be desired to
@@ -556,7 +612,7 @@ has 'choose_retries' => (
 	is        => 'ro',
 	isa       => 'Int',
 	lazy      => 1,
-	default   => sub { my $self = shift; $self->_has_should_choose_retry ? 1 : 0 }
+	default   => 1,
 );
 
 has '_chosen_retries' => (
